/*
*****Reginald Client GUI Controller*****
  Created by Nicholas Williams
  This source code can be run with the Processing IDE and is a great
  full-featured GUI in controlling Reginald: a remote controlled surveillance bot
  The client program is what actually implements many of Reginald's features
*/
import hypermedia.net.*;                    //UDP library
import controlP5.*;                         //Library for creating the GUI
ControlP5 cp5;                              //GUI class; make an instance
UDP udp;
//Various variables that I declare globally
int myColorBackground = color(100);    //color of the background
String myString;                       //code doesn't work unless this is global; too lazy to figure out why, easier just to stick it here
boolean keyLocked;                     //checks whether the Locking Switch is on, disables keys
boolean clearFirst=true;               //helps in sending a clear command
boolean[] keys = new boolean[526];
int tiltAngle = 0;                     //initializes where the tilt angle of Reginald starts at
int panAngle = 0;
String ip;          
int port;
boolean speedLock = false;             //switches between velocity and position control
int speedLockSpeed=0;                  //starting velocity
int misslesFired = 0;

//Instances of buttons and widgets from ControlP5
Tab settings;
Textarea console;
Textfield comField, ipAddress, portNumber;
Button upButton, downButton, rightButton, leftButton, wButton, aButton, sButton, dButton, space, resetMissles;
Toggle velPosToggle, lockingSwitch, lightSwitch;
Textlabel toggleLabel, velPosLabel, lightLabel, missleLabel;
Slider sliderSpeed, sliderVelocity;
Knob angleKnob, panKnob;
RadioButton firePattern;
PImage turretOne, turretTwo;

void setup() {
 size(500,300);     //size of window
 smooth();          //makes things look fancy
 noStroke();        //I have no idea what this does... I'm pretty sure I need it
 udp = new UDP(this, 6000);
 udp.listen(true);
 cp5 = new ControlP5(this); 

//Buttons and widgets; created by the ControlP5 library
settings = cp5.addTab("Settings").setColorLabel(color(255)).setColorActive(color(255,128,0));
cp5.getTab("Settings").activateEvent(true).setId(2); 
console = cp5.addTextarea("txt").setPosition(5,20).setSize(150, 255).setFont(createFont("proggy",9)).setColor(color(255,128,0)).setColorBackground(color(0)); 
comField = cp5.addTextfield("Send Message Text").setPosition(5,280).setSize(110, 18).setFont(createFont("proggy",9)).setColor(color(255,0,0)); 
ipAddress = cp5.addTextfield("ipAddress").setPosition(100,100).setSize(110, 18).setFont(createFont("proggy",9)).setColor(color(255,128,0)).setColorBackground(color(0)).setValue("192.168.1.177");
portNumber = cp5.addTextfield("portNumber").setPosition(100,140).setSize(110, 18).setFont(createFont("proggy",9)).setColor(color(255,128,0)).setColorBackground(color(0)).setValue("8888");    
cp5.addBang("Send").setPosition(120, 280).setSize(35, 18).getCaptionLabel().align(ControlP5.CENTER, ControlP5.CENTER);  
space = cp5.addButton("space").setPosition(185, 250).setSize(285, 25); 
upButton = cp5.addButton("up").setPosition(417, 184).setSize(25,25).setImages(loadImage("UPUn.PNG"), loadImage("UPOver.PNG"), loadImage("UPPress.PNG")); 
downButton = cp5.addButton("down").setPosition(417, 212).setSize(25,25).setImages(loadImage("DOWNUn.PNG"), loadImage("DOWNOver.PNG"), loadImage("DOWNPress.PNG"));
rightButton = cp5.addButton("right").setPosition(445, 212).setSize(25,25).setImages(loadImage("RIGHTUn.PNG"), loadImage("RIGHTOver.PNG"), loadImage("RIGHTPress.PNG")); 
leftButton = cp5.addButton("left").setPosition(389, 212).setSize(25,25).setImages(loadImage("LEFTUn.PNG"), loadImage("LEFTOver.PNG"), loadImage("LEFTPress.PNG")); 
wButton = cp5.addButton("w").setPosition(213, 184).setSize(25,25).setImages(loadImage("WUn.PNG"), loadImage("WOver.PNG"), loadImage("WPress.PNG")); 
sButton = cp5.addButton("s").setPosition(213, 212).setSize(25,25).setImages(loadImage("SUn.PNG"), loadImage("SOver.PNG"), loadImage("SPress.PNG"));
dButton = cp5.addButton("d").setPosition(241, 212).setSize(25,25).setImages(loadImage("DUn.PNG"), loadImage("DOver.PNG"), loadImage("DPress.PNG"));
aButton = cp5.addButton("a").setPosition(185, 212).setSize(25,25).setImages(loadImage("AUn.PNG"), loadImage("AOver.PNG"), loadImage("APress.PNG")); 
resetMissles = cp5.addButton("resetMissles").setPosition(300,140).setSize(40,20).setLabelVisible(false);
lightSwitch = cp5.addToggle("lightSwitch").setPosition(440,280).setSize(30,18).setValue(false);
lightLabel = cp5.addTextlabel("lightLabel").setPosition(365,285).setValueLabel("LIGHT SWITCH");
missleLabel = cp5.addTextlabel("missleLabel").setPosition(340, 145).setValueLabel("RESET MISSLES");
lockingSwitch = cp5.addToggle("LockingSwitch").setPosition(185,280).setSize(30,18).setValue(true);
toggleLabel = cp5.addTextlabel("ToggleLabel").setPosition(218,285).setValueLabel("LOCKING SWITCH");
velPosLabel = cp5.addTextlabel("VelocityLabel").setPosition(340,105).setValueLabel("VELOCITY CONTROL ON");   
sliderSpeed = cp5.addSlider("speed").setPosition(185, 30).setRange(0,90).setNumberOfTickMarks(91).snapToTickMarks(true).showTickMarks(false);  
sliderVelocity = cp5.addSlider("Velocity").setPosition(185, 42).setRange(-90,90).setValue(0).setNumberOfTickMarks(181).snapToTickMarks(true).showTickMarks(false); 
angleKnob = cp5.addKnob("AngleKnob").setRange(0,150).setPosition(380,25).setRadius(20).setViewStyle(Knob.ARC).setDragDirection(Knob.VERTICAL); 
velPosToggle = cp5.addToggle("VelocityControl").setPosition(300,100).setValue(false).setLabelVisible(false);
panKnob = cp5.addKnob("panKnob").setRange(0,180).setPosition(440,25).setRadius(20).setViewStyle(Knob.ARC).setDragDirection(Knob.VERTICAL);
//Arrange settings widgets to the settings tab; make console global
cp5.getController("resetMissles").moveTo("Settings");
cp5.getController("VelocityControl").moveTo("Settings");
cp5.getController("VelocityLabel").moveTo("Settings");
cp5.getController("ipAddress").moveTo("Settings");
cp5.getController("portNumber").moveTo("Settings");
cp5.getController("missleLabel").moveTo("Settings");
 
turretOne = loadImage("Turret.PNG");
turretTwo = loadImage("Turret.PNG");
}

/*Program loops through this draw loop. When I send a command to Reginald and then
stop, this code will send a clear command to Reginald so that Reginald stops moving. This code
checks to see if this is the first clear command since a command has been made, so that way Reginald does
not get flooded with clear commands that won't do anything after the first one anyway. Also it checks for
key press; without this, if a user holds down a key, this program will send alternating commands between
the clear command and the command associated with the key press.
*/
void draw() {
  background(myColorBackground);
  if(!settings.isActive()){
    image(turretOne, 257, 100);
    image(turretTwo, 337, 100);
  }  
  if(clearFirst==true&&!keyPressed&&!speedLock){   
    packageAndSend("clear", 0);
    clearFirst=false;
  }
}


/*Takes in a string value and a number. Whenever the program needs to send a packet to the Arduino, it directs
itself here. The string is the domain of what is being changed. The parameter is the how the Arduino should
behave in the domain the program specifies. For example; when the program receives an "up" event, the
program will direct itself here and send "froba00040" assuming that the speed is set to 40. The Arduino sees
this packet, and knows it needs to move forward at at value of 129 (89 is stationary).
Reginald is set up to understand only 10 char strings, hence the extra zeros
*/

void packageAndSend(String address, int parameter) {
  String stringParam = ""+parameter;         //convert parameter to string
  int i = address.length();
  int j = stringParam.length();
  int k = 10 - i - j;                        //figure out how many zeros program needs
  String zeroString = "";
  int counter = 0;
  while(counter < k){
    zeroString = zeroString + "0";
    counter++;
  }
  String finalString = address + zeroString + stringParam;   //throw everything together; 10 char in length
  ip = ipAddress.getText();
  port = int(portNumber.getText());
  udp.send(finalString, ip, port);          //send this bad boy to the Arduino
}


//If user presses a key, come here

void keyPressed() {
  if(!keyLocked){      //If the Locking Switch is on; move on with life
    keys[keyCode] = true;
    if(checkKey(UP)){up();}
    else if(checkKey(DOWN)){down();}
    else if(checkKey(LEFT)){left();}
    else if(checkKey(RIGHT)){right();}
    else if(key == ' '){space();}
    else if(key == 'w'){w();}
    else if(key == 'a'){a();}
    else if(key == 's'){s();}
    else if(key == 'd'){d();}
    else if(key == 'l'){lightSwitch.toggle();}
    else {}
  }else{ }
}
/*Controls events that occur within the sending text box. Created to send custom commands to Reginald.
Also is used for entering in passwords*/
public void Send() {
  String txtSend = comField.getText();
  addToConsole(txtSend);
  if(txtSend.length()<10){packageAndSend(txtSend, 0);}
  else{                               //Don't go to packageAndSend if it's already 10 char long
    ip = ipAddress.getText();
    port = int(portNumber.getText());
    udp.send(txtSend, ip, port);
    }  
  console.scroll(1);                  //if the console is scrolling, this keeps it at the bottom
  comField.clear();                   //clear the sending text box
}
//If message is recieved from Reginald, convert the message to string and display in the console
void receive( byte[] data){
  for(int i=0; i<data.length; i++){
    myString = new String(data);                  //convert the data to string
  }
  addToConsole(myString);             
}
//Move Reginald forward; check to see if "up" controls velocity or position
void up() {
  int speed;
  boolean leftKey = checkKey(LEFT);
  boolean rightKey = checkKey(RIGHT);   
  if(speedLock){speedLockSpeed++; sliderVelocity.setValue(speedLockSpeed); speed = speedLockSpeed + 89;   
  }else{speed = 89 + int(sliderSpeed.getValue());}
  if(leftKey){packageAndSend("left",speed);}
  else if(rightKey){packageAndSend("right",speed);}
  else{packageAndSend("froba", speed);} 
  clearFirst=true; 
}
//Move Reginald backwards; check to see if "down" controls velocity or position
void down(){
  int speed;
  boolean leftKey = checkKey(LEFT);
  boolean rightKey = checkKey(RIGHT); 
  if(speedLock){speedLockSpeed--; sliderVelocity.setValue(speedLockSpeed); speed = speedLockSpeed + 89;
  }else{speed = 89 - (int(sliderSpeed.getValue()));}
  if(leftKey){packageAndSend("left",speed);}
  else if(rightKey){packageAndSend("right",speed);}  
  else{packageAndSend("froba", speed);}
  clearFirst=true; 
}
//15 is the turn rate; sends clear command so Reginald doesn't keep spinning after command
void right() {if(!speedLock){packageAndSend("turn", 15); clearFirst=true;}else{packageAndSend("right",speedLockSpeed+89);addToConsole("turning right");}}    
//-15 turns Reginald the other way
void left() {if(!speedLock){packageAndSend("turn", -15); clearFirst=true;}else{packageAndSend("left",speedLockSpeed+89);addToConsole("turning left");}}
//increment tilt angle; set knob to tilt value; send data to Reginald
void w() {if(tiltAngle<150){tiltAngle++;}else{tiltAngle=150; addToConsole("You reached the upper tilt limit");} angleKnob.setValue(tiltAngle); packageAndSend("angle", tiltAngle);}
//pan turret and camera to the left
void a() {if(panAngle>0){panAngle--;}else{panAngle=0; addToConsole("You reached the left pan limit");} panKnob.setValue(panAngle); packageAndSend("pan", panAngle);}
//decrement tilt angle; set knob to tilt value; send data to Reginald
void s() {if(tiltAngle>0){tiltAngle--;}else{tiltAngle=0; addToConsole("You reached the lower tilt limit");} angleKnob.setValue(tiltAngle); packageAndSend("angle", tiltAngle);}
//pan turret and camera to the right
void d() {if(panAngle<180){panAngle++;}else{panAngle=180; addToConsole("You reached the right pan limit");} panKnob.setValue(panAngle); packageAndSend("pan", panAngle);}
//Tell Reginald to fire a missle
void space() {if(misslesFired%2==0){packageAndSend("fire", 0);}else{packageAndSend("fire", 1);}markTurret();}
/*If Locking switch is active, then keyboard keys won't send command. This is very useful when
the user wants to enter a command in the console. That the key strokes to enter the command do
not send commands themselves*/
void LockingSwitch(boolean theFlag) {if(theFlag==true){keyLocked = true; addToConsole("Keys are locked");}else{keyLocked = false; addToConsole("Keys enabled");}}
//Use the angle knob to set angle, get the value and send it to Reginald
void AngleKnob(){tiltAngle = int(angleKnob.getValue()); if(!keyPressed){packageAndSend("angle", tiltAngle);}}
//Button in settings that indicates we can fire more missles!
void resetMissles(){misslesFired = 0; turretOne = loadImage("Turret.PNG"); turretTwo = loadImage("Turret.PNG"); addToConsole("Missles reset.");}
//function that sends a command to Reginald to either turn his spotlight on or off
void lightSwitch(boolean theFlag) {if(theFlag==true){addToConsole("Light activated"); packageAndSend("light", 1);}else{addToConsole("Light deactivated"); packageAndSend("light", 0);}}
//Use the pan knob to set the panning angle, get the value and send it to Reginald
void panKnob() {panAngle = int(panKnob.getValue()); if(!keyPressed){packageAndSend("pan", panAngle);}}
/*The Velocity Control On toggle changes the up and down keys to represent velocity or position
This is a nice feature because in cases of long straight stretches, the user could set the up and down
keys to control the velocity. Then the user can allow Reginald to automonously move forward/backward 
while the user can control the camera and turret. This allows for a "strafe" style movement.
Position control is available for fine control of Reginald*/
void VelocityControl(boolean Flag) {
  if(Flag==true){speedLock=true; addToConsole("Velocity Locked");}
  else{speedLock=false;speedLockSpeed=0;sliderVelocity.setValue(speedLockSpeed); addToConsole("Position enabled");}
} 
//Simply replaces the turret icon with an appropriate one when a missle is fired
void markTurret(){
  if(misslesFired==0){turretOne = loadImage("Turret1.PNG");addToConsole("Launching missle 1...");}
  else if(misslesFired==1){turretTwo = loadImage("Turret1.PNG");addToConsole("Launching missle 2...");}
  else if(misslesFired==2){turretOne = loadImage("Turret2.PNG");addToConsole("Launching missle 3...");}
  else if(misslesFired==3){turretTwo = loadImage("Turret2.PNG");addToConsole("Launching missle 4...");}
  else if(misslesFired==4){turretOne = loadImage("Turret3.PNG");addToConsole("Launching missle 5...");}
  else if(misslesFired==5){turretTwo = loadImage("Turret3.PNG");addToConsole("Launching missle 6...");}
  else if(misslesFired==6){turretOne = loadImage("Turret4.PNG");addToConsole("Launching missle 7...");}
  else if(misslesFired==7){turretTwo = loadImage("Turret4.PNG");addToConsole("Launching last missle...");}
  else{addToConsole("No missles; reset the missles.");}
  misslesFired++;
}
//A simple function that adds text to the console
void addToConsole(String stringToAdd){String textInBox = console.getText(); console.setText(textInBox + "\n " + stringToAdd); console.scroll(1);}
boolean checkKey(int k)
{
  if (keys.length >= k) {
    return keys[k];  
  }
  return false;
}
void keyReleased()
{ 
  keys[keyCode] = false; 
}
