#include <SPI.h>
#include <Ethernet.h>
#include <EthernetUdp.h>   
#include <Servo.h>
#include <stdlib.h>

byte mac[] = {0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED };
IPAddress ip(192,168,1,177);
unsigned int localPort = 8888;
char packetBuffer[UDP_TX_PACKET_MAX_SIZE];
EthernetUDP Udp;

Servo servoOne;         //Right wheel servo
Servo servoTwo;         //Left wheel servo
Servo servoThree;       //Panning servo
Servo servoFour;        //Tilt angle servos

void setup(){
  Ethernet.begin(mac,ip);
  Udp.begin(localPort);

  Serial.begin(9600);
  
  servoOne.attach(3);
  servoTwo.attach(5);
  servoThree.attach(6);
  servoFour.attach(9);

  servoOne.write(89);      //Wheel servos are continuous, 89 is a stopped position
  servoTwo.write(89);
  servoThree.write(89);
  servoFour.write(75);    //I limited the Angle of panning to 150 degrees, hence choosing 75 
  
  pinMode(7, OUTPUT);          //Turret DC motor 1
  pinMode(2, INPUT);          //Turret switch 1
  pinMode(8, OUTPUT);          //Turret DC motor 2
  pinMode(4, INPUT);          //Turret switch 2
  pinMode(A5, OUTPUT);        //For the flashlight
  Salutations(); //verify USER to use Reginald, if you want to bypass the encryption, simply comment this out.
}

//Don't do anything until a packet is received
void loop(){
  int packetSize = Udp.parsePacket();
  if(packetSize){
    Udp.read(packetBuffer,UDP_TX_PACKET_MAX_SIZE);
    String myString(packetBuffer);
    splitAndDirect(myString);
  }
}

/*splitAndDirect: used for accepting a packet, spliting the destination from the 
parameters, setting the parameters as an integer, and delivering that integer to
the appropriate desination
The "first half" is essentially the domain the packet regards, and the "second half"
is a parameter of what to do in that domain*/
void splitAndDirect(String packet) {
  Serial.println(packet);
  int parameter=0;
  String firstHalf = packet.substring(0,5);
  String secondHalf = packet.substring(5,10);
  for(int i=0; i<5; i++){
   if(secondHalf.charAt(i)=='0'){ 
   } else {
     String secondHalf2 = secondHalf.substring(i,5);
     char tmpArray[secondHalf2.length() + 1];
     secondHalf2.toCharArray(tmpArray, sizeof(tmpArray));
     parameter = atoi(tmpArray);
     break;
   }
  }
  int par = parameter;
  if(firstHalf=="froba"){frontBack(par);}                  //Move forward or backward
  else if(firstHalf=="left0"){turningLeft(par);}           //Turn left while driving forward (Not fully implemented yet)
  else if(firstHalf=="right"){turningRight(par);}          //Turn right while driving forward (Not fully implemented yet)
  else if(firstHalf=="turn0"){turnLeftRight(par);}         //Pivot in place, parameter says which direction
  else if(firstHalf=="angle"){tiltAngle(par);}             //Tilt the camera and turrets
  else if(firstHalf=="pan00"){panLeftRight(par);}          //Pan the camera and turrets
  else if(firstHalf=="light"){flashLight(par);}            //Toggle the flashlight
  else if(firstHalf=="fire0"){fireMissle(par);}            //Fire a missile, parameter says which side to fire from
  else if(firstHalf=="clear"){clearValues();}              //Stop the wheel servos
}

void frontBack(int par){
  Serial.print("froba ");
  Serial.println(par);
  servoOne.write(par);
  servoTwo.write(180 - par);
}
void turnLeftRight(int par){
  Serial.print("Turn left right ");
  Serial.println(par);
  servoOne.write(90-par);
  servoTwo.write(90-par);
}
void tiltAngle(int par){
  Serial.print("angle ");
  Serial.println(par);
  servoFour.write(par);
}
void panLeftRight(int par){
  Serial.print("pan left right ");
  Serial.println(par);
  servoThree.write(par);
}
void flashLight(int par){
  
  if(par){
    digitalWrite(A5, HIGH);
    Serial.println("HIGH");
  } else{
    digitalWrite(A5, LOW);
    Serial.println("LOW");
  }
}

void fireMissle(int par){
  int turretPin;
  int switchPin;
  if(par == 0){
    turretPin = 7;
    switchPin = 2;
  }else{
    turretPin = 8;
    switchPin = 4;
  }
  digitalWrite(turretPin, HIGH);
  boolean flag = true;
  while(flag){
    if(digitalRead(switchPin)==HIGH){
     Serial.println("Fired turret"); 
     flag = false; 
    }
    Serial.println("checking");
    delay(5);
  }
  delay(1700);
  digitalWrite(turretPin,LOW);
  Serial.println("FIRE!");
}
void clearValues(){
  Serial.println("clear");
  servoOne.write(89);
  servoTwo.write(89);
  //servoThree.write();
}
void turningLeft(int par){
  int par2 = (par-90) * .7;
  Serial.print(par);
  Serial.print("   ");
  Serial.println(par2);
  servoOne.write(par);
  servoTwo.write(90 - par2);
}
void turningRight(int par){
  int par2 = (par-90) * .7;
  Serial.print(par);
  Serial.print("   ");
  Serial.println(par2);
  servoTwo.write(par);
  servoOne.write(90 - par2);
}

/*A function to encrypt Reginald. USER cannot access Reginald until USER enters in
the wake command and password. Reginald will check for a wake command every 15 
seconds. When the command is sent, Reginald will send back a greeting and request
from the USER a password. Reginald then waits for 15 additional seconds and checks
to see if the correct password was entered. If not, Reginald will stay in his while
loop. If the correct password is entered, the USER will break from the while loop
and continue to ultilize Reginald's capabilities.*/

void Salutations() {
 
  boolean flag = true;
  boolean flag2 = true;
  while(flag){     //stuck in this while loop until USER enters password
    Serial.println("Here");
    delay(1000);        //checks to see if USER sent wake command
    int packetSize = Udp.parsePacket();
    if(packetSize)
    {
      Udp.read(packetBuffer, UDP_TX_PACKET_MAX_SIZE);
      String wakeUp(packetBuffer);
      Serial.println(wakeUp);
      if(wakeUp == "wake000000") {     //did the USER send a wake command?
        Udp.beginPacket(Udp.remoteIP(), Udp.remotePort());
        //send the user a greeting; request a password
        Udp.write("Hello USER, I am named Reginald; I was constructed the 23rd of July, 2012 at 22:31 by Nicholas F Williams. I am a remotely controlled surveillance bot. A password is required to operate me; what is my password?");
        Udp.endPacket();
        while(flag2){
          delay(15000);  //wait for the USER to enter a password
          int packetSize2 = Udp.parsePacket();
          if(packetSize2)
          {
            Udp.read(packetBuffer, UDP_TX_PACKET_MAX_SIZE);
            Serial.println("The password you entered is");
            String password(packetBuffer);
            Serial.println(password);
            if(password == "password0")       //"password" is password, the GUI adds a "0" to the end of the packet because it's 9 char long
            {
              Udp.beginPacket(Udp.remoteIP(), Udp.remotePort());
              Udp.write("Welcome; you now have access.");       
              Udp.endPacket(); 
              flag = false;
              flag2 = false;       // to break from the while(flag) loop
            }else                 //incorrect password or no password
            {
              Udp.beginPacket(Udp.remoteIP(), Udp.remotePort());
              Udp.write("You cannot be confirmed; access denied.");         
              Udp.endPacket();
            }
          }
        }
      }
    }
  }
}

